package com.example.aped;

import android.Manifest;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.MenuItem;

import android.view.Menu;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.example.aped.communication.Communicator;
import com.example.aped.utils.ExternalStorageHandler;
import com.example.aped.utils.IFAVORITES;
import com.example.aped.utils.IXML;
import com.example.aped.utils.TestFavorites;
import com.example.aped.utils.TestXML;
import com.example.aped.utils.XMLHandler;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.navigation.NavigationView;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;

import static android.content.pm.PackageManager.PERMISSION_GRANTED;

public class MainActivity extends AppCompatActivity {
    /** permission code for the storage permisson.*/
    private final int STORAGE_PERMISSION_CODE = 42;
    /** was soll angezeigt werden in Navigation.*/
    private AppBarConfiguration mAppBarConfiguration;
    /** zur Verwendung von xml anstatt der direkten Einbindung.*/
    public  IXML xml;
    /** zur Verwendung von uebergabe anstatt der direkten Einbindung.*/
    public Communicator uebergabe;
    /** zur Verwendung von favorite anstatt der direkten Einbindung.*/
    public  IFAVORITES favorite = new TestFavorites();
    /** allgemeines.*/
    private String ip = "192.168.1.220";
    /**
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //checks that the permission to read and write the xml is granted
        ensurePermissions();
        setXML();
        uebergabe = new Communicator(ip, 8080, this);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = findViewById(R.id.drawer_layout);
        NavigationView navigationView = findViewById(R.id.nav_view);
        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_all_IOs, R.id.nav_your_IOs)
                .setDrawerLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this,
                                                        R.id.nav_host_fragment);
        NavigationUI.setupActionBarWithNavController(this,
                                        navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);
        //xml = new TestXML();
    }
    /** Fügt Elemente zur Aktionsleiste hinzu, wenn diese vorhanden ist.*/
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    /**Navigation einrichten.*/
    @Override
    public boolean onSupportNavigateUp() {
        NavController navController = Navigation.findNavController(this,
                                                        R.id.nav_host_fragment);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }
    /**was soll bei click auf den jeweiligen itembutton passieren.*/
    @Override
    public boolean onOptionsItemSelected(final @NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_set_IP:
                MaterialAlertDialogBuilder alert = new
                        MaterialAlertDialogBuilder(MainActivity.this);
                alert.setTitle(R.string.action_set_IP);
                alert.setMessage("Please enter the ip address of the "
                        + "device to be connected");
                final EditText input = new EditText(MainActivity.this);
                input.setText(ip);
                LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        LinearLayout.LayoutParams.MATCH_PARENT);
                input.setLayoutParams(lp);
                alert.setView(input);
                alert.setPositiveButton("connect",
                        new DialogInterface.OnClickListener() {
                            public void onClick(final DialogInterface dialog,
                                                final int which) {
                                // Write your code here to execute after dialog
                                ip = input.getText().toString();
                                Toast.makeText(getApplicationContext(),
                                        ip + " IP connect",
                                        Toast.LENGTH_SHORT).show();
                                uebergabe = new Communicator(ip, 8080,
                                        MainActivity.this);
                            }
                        });
                alert.setNegativeButton("exit",
                        new DialogInterface.OnClickListener() {
                            public void onClick(final DialogInterface dialog,
                                                final int which) {
                                // Write your code here to execute after dialog
                                Toast.makeText(getApplicationContext(),
                                        "exit clicked",
                                        Toast.LENGTH_SHORT).show();
                                dialog.cancel();
                            }
                        });
                alert.show();
                break;

            case R.id.action_adjust_xml:
                Toast.makeText(this, "Adjust .xml",
                        Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(Intent.ACTION_SEND);
                String shareBody = "gegen xml für den PI ersetzen";
                intent.setType("text/plain");
                intent.putExtra(intent.EXTRA_TEXT, shareBody);
                startActivity(intent);
                /**
                 * xml anstatt beispieltext muss übergeben werden
                 * rückgabe der xml fehlt noch
                 */
                break;

            case R.id.action_download_xml:
                    Toast.makeText(this, "Downloaded .xml",
                        Toast.LENGTH_SHORT).show();
                break;
            case R.id.action_upload_xml:
                    Toast.makeText(this, "Upload .xml",
                        Toast.LENGTH_SHORT).show();
                break;
            default:
                Log.e("MainActivity", "unknown item: "
                        + item.toString());
                return super.onOptionsItemSelected(item);
        }
        return true;
    }

    /**
     * ensures that all the permissions needed by the application are granted.
     */
    private void ensurePermissions() {
        if (ContextCompat.checkSelfPermission(
                        this,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PERMISSION_GRANTED) {
            if (ActivityCompat.shouldShowRequestPermissionRationale(
                    this,
                    Manifest.permission.READ_EXTERNAL_STORAGE)) {

                new AlertDialog.Builder(this)
                    .setTitle("Permission needed")
                    .setMessage("This permission is needed to access the "
                        + "XML configuration file")
                    .setPositiveButton("ok", (dialog, which) ->
                        ActivityCompat.requestPermissions(
                            MainActivity.this,
                            new String[]{
                                Manifest.permission.WRITE_EXTERNAL_STORAGE},
                                    STORAGE_PERMISSION_CODE))
                    .setNegativeButton("cancel", (dialog, which) ->
                        System.exit(1))
                    .create()
                    .show();

            } else {
                ActivityCompat.requestPermissions(
                    this,
                    new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                    STORAGE_PERMISSION_CODE);
            }
        }
    }

    /**
     * handles request permission results.
     * @param requestCode the code of the requested
     * @param permissions the permissions requested
     * @param grantResults the result of the permission request
     */
    @Override
    public void onRequestPermissionsResult(
            final int requestCode,
            @NonNull final String[] permissions,
            @NonNull final int[] grantResults) {
        if (requestCode == STORAGE_PERMISSION_CODE) {
            if (grantResults.length > 0
                    && grantResults[0] == PERMISSION_GRANTED) {
                Toast.makeText(
                    this,
                    "Permission GRANTED",
                    Toast.LENGTH_LONG)
                    .show();
            } else {
                System.exit(1);
            }
        }
    }

    private void setXML() {
        try {
            String xmlPath = ExternalStorageHandler.
                    getExternalPrivateStorageDir(MainActivity.this);
            File xmlFile = new File(xmlPath, "config.xml");

            //create the default XML config by using the default.xml from the assets folder
            if (xmlFile.exists()) {
                xmlFile.delete();
            }
            InputStream inputStream = getAssets().open(
                    "XML/Test.xml");
            byte[] buffer = new byte[inputStream.available()];
            inputStream.read(buffer);
            inputStream.close();
            FileOutputStream fileOutputStream = new FileOutputStream(xmlFile);
            fileOutputStream.write(buffer);
            fileOutputStream.close();

            xml = new XMLHandler(xmlFile);
        } catch (FileNotFoundException e) {
            Log.e("Main Activity", "Exception while reading the "
                    + "external private storage dir: " + e.getMessage());
        } catch (ParserConfigurationException | IOException | SAXException e) {
            Log.e("Main Activity", "XMLHandler failed to initialise: "
                    + e.getMessage());
        }
    }
}
